/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.dql.connector;

import org.eclipse.core.runtime.IProgressMonitor;

public class EclipseProgressListener implements QueryProgressListener {
	private static int SECONDSPERDAY = 86400;
	private static int SECONDSPERHOUR = 3600;
	private static int SECONDSPERMINUTE = 60;
	private float progressSoFar;
	private String subTask;
	private int remainingTime;

	private IProgressMonitor monitor;

	public EclipseProgressListener(IProgressMonitor monitor) {
		this.monitor = monitor;
		progressSoFar = 0.0f;
		remainingTime = -1;
		monitor.beginTask("Executing query", 10000);
	}

	@Override
	public void updateSubTask(String subTask) {
		if (!isCanceled()) {
			this.subTask = subTask;
			updateSubTaskText();
		}
	}

	@Override
	public void updateProgress(float progressInPercent) {
		if (!isCanceled()) {
			float actualProgress = progressInPercent - progressSoFar;
			int progressAsInt = (int) (actualProgress * 100.0f);
			monitor.worked(progressAsInt);
			progressSoFar = progressInPercent;
		}
	}

	@Override
	public void updateRemainingTime(int seconds) {
		// TODO remainingTime use some nice format!
		if (!isCanceled()) {
			remainingTime = seconds;
			updateSubTaskText();
		}
	}

	private void updateSubTaskText() {
		if (!isCanceled()) {
			String remainingTimeString = "" + format(remainingTime);
			if (remainingTime == -1) {
				remainingTimeString = "calculating...";
			}
			monitor.subTask("Current task: " + subTask 
					+ "\nRemaining time: " + remainingTimeString);
		}
	}

	private String format(int seconds) {
		int days = 0;
		int hours = 0;
		int minutes = 0;
		
		if (seconds/SECONDSPERDAY >= 1) {
			days = seconds/SECONDSPERDAY;
		}
		seconds = seconds % SECONDSPERDAY;
		
		if (seconds/SECONDSPERHOUR >= 1) {
			hours = seconds/SECONDSPERHOUR;
		}
		seconds = seconds % SECONDSPERHOUR;
		
		if (seconds/SECONDSPERMINUTE >= 1) {
			minutes = seconds/SECONDSPERMINUTE;
		}
		seconds = seconds % SECONDSPERMINUTE;
		
		return format(days, hours, minutes, seconds);
	}
	
	/*
	 * We check for > 200 since maxint equals 270 days, and the counter looks like it's
	 * stuck, not that this case ever matters :-P
	 */
	private String format(int days, int hours, int minutes, int seconds) {
		String ret = "";
		if (days == 1)
			ret += days + " Tag ";
		if (days > 1)
			ret += days + " Tage ";
		if (hours == 1)
			ret += hours + " Stunde ";
		if (hours > 1)
			ret += hours + " Stunden ";
		if (minutes == 1)
			ret += minutes + " Minute ";
		if (hours > 1)
			ret += minutes + " Minuten ";
		if (seconds == 1)
			ret += seconds + " Sekunde ";
		if (seconds > 1)
			ret += seconds + " Sekunden ";
		if (ret.equals(""))
			ret = "0 Sekunden";
		if (days > 200)
			return "mehr als 200 Tage";
		return ret;
	}

	private boolean isCanceled() {
		// TODO look into
		if (monitor.isCanceled()) {
			monitor.subTask("Cancelling...");
			Thread.currentThread().interrupt();
			return true;
		} else {
			return false;
		}
	}
}