/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.dql.connector;

import tools.descartes.dql.models.mapping.mapping.EntityMapping;

/**
 * This interface defines the necessary methods to execute queries for model
 * structure-related information. The information retrieved can be used to form
 * queries of a higher order, e.g. to retrieve performance metrics.
 * 
 * @author Fabian Gorsler <fabian.gorsler@student.kit.edu>
 */
public interface ModelStructureQueryConnector extends QueryConnector {
	public EntityMapping requestDoF(EntityMapping request);

	/**
	 * Request the connector to resolve all entities from a performance model
	 * instance and apply an optional filter condition
	 * 
	 * @param request
	 *            A mapping model instance with optional filters
	 * @return The mapping model instance with all resolved entities attached
	 */
	public EntityMapping requestEntities(EntityMapping request);

	/**
	 * Request the connector to resolve all computable probes for the entities
	 * attached to this mapping model instance
	 * 
	 * @param request
	 *            A mapping model instance with performance model instance
	 *            entities
	 * @return The mapping model instance with computable probes attached to the
	 *         model entities
	 */
	public EntityMapping requestProbes(EntityMapping request);

	 //TODO
	public boolean supportsDoFRequests();

	/**
	 * Test whether entity requests are supported by the implementing class
	 * 
	 * @return True if entity requests are supported, false otherwise
	 */
	public boolean supportsEntityRequests();

	/**
	 * Test whether probe requests are supported by the implementing class
	 * 
	 * @return True if probe requests are supported, false otherwise
	 */
	public boolean supportsProbeRequests();
}
