package tools.descartes.dql.core.engine.whatif;

import java.util.Iterator;

import tools.descartes.dql.models.mapping.domain.DecimalResult;
import tools.descartes.dql.models.mapping.domain.Probe;
import tools.descartes.dql.models.mapping.domain.Resource;
import tools.descartes.dql.models.mapping.domain.Service;
import tools.descartes.dql.models.mapping.mapping.EntityMapping;

public class DoubleFilter implements WhatIfFilter {

	private boolean modifier;
	private double filteringDouble;

	public DoubleFilter(double filteringDouble, boolean modifier) {
		this.filteringDouble = filteringDouble;
		this.modifier = modifier;
	}

	@Override
	public EntityMapping applyFilter(final EntityMapping initial, final EntityMapping modified) {
		if (!initial.equals(modified)) { // else the initial gets modified and
											// looses all content
			// checking all Resouces
			for (Iterator<Resource> resourceIterator = modified.getResources().iterator(); resourceIterator.hasNext();) {
				Resource resource = resourceIterator.next();
				for (Iterator<Probe> ProbeIterator = resource.getProbes().iterator(); ProbeIterator.hasNext();) {
					Probe probe = ProbeIterator.next();
					DecimalResult modifiedResult = (DecimalResult) probe;
					DecimalResult initialResult = getResult(initial, resource.getIdentifier(), probe.getMetricName());

					if (modifier) {
						if (filterWithModifier(modifiedResult, initialResult))
							ProbeIterator.remove();

					} else if (filterWithoutModifier(modifiedResult, initialResult))
						ProbeIterator.remove();
				}

				if (resource.getProbes().isEmpty())
					resourceIterator.remove();
			}
			// checking all Services
			for (Iterator<Service> serviceIterator = modified.getServices().iterator(); serviceIterator.hasNext();) {
				Service service = serviceIterator.next();
				for (Iterator<Probe> probeIterator = service.getProbes().iterator(); probeIterator.hasNext();) {
					Probe probe = probeIterator.next();
					DecimalResult modifiedResult = (DecimalResult) probe;
					DecimalResult initialResult = getResult(initial, service.getIdentifier(), probe.getMetricName());

					if (modifier) {
						if (filterWithModifier(modifiedResult, initialResult))
							probeIterator.remove();

					} else if (filterWithoutModifier(modifiedResult, initialResult))
						probeIterator.remove();

				}

				if (service.getProbes().isEmpty())
					serviceIterator.remove();
			}

			return modified;
		} else
			return initial;

	}

	private boolean filterWithoutModifier(final DecimalResult modifiedResult, final DecimalResult initialResult) {
		if (Math.abs(initialResult.getValue().doubleValue() - modifiedResult.getValue().doubleValue()) < filteringDouble)
			return true;
		else
			return false;
	}

	private boolean filterWithModifier(final DecimalResult modifiedResult, final DecimalResult initialResult) {
		double percent = Math.abs((modifiedResult.getValue().doubleValue() / initialResult.getValue().doubleValue()) - 1);
		if (percent < filteringDouble || modifiedResult.getValue().intValue() == 0)
			return true;
		else
			return false;

	}

	private DecimalResult getResult(final EntityMapping initial, String identifier, String metricIdentifier) {
		for (Iterator<Resource> resourceIterator = initial.getResources().iterator(); resourceIterator.hasNext();) {
			Resource resource = resourceIterator.next();
			if (resource.getIdentifier().equals(identifier)) {
				for (Iterator<Probe> probeIterator = resource.getProbes().iterator(); probeIterator.hasNext();) {
					Probe probe = probeIterator.next();
					if (probe.getMetricName().equals(metricIdentifier)) {
						return (DecimalResult) probe;
					}
				}
			}
		}

		for (Iterator<Service> serviceIterator = initial.getServices().iterator(); serviceIterator.hasNext();) {
			Service service = serviceIterator.next();
			if (service.getIdentifier().equals(identifier)) {
				for (Iterator<Probe> probeIterator = service.getProbes().iterator(); probeIterator.hasNext();) {
					Probe probe = probeIterator.next();
					if (probe.getMetricName().equals(metricIdentifier)) {
						return (DecimalResult) probe;
					}
				}
			}
		}
		return null;
	}

}
