

package tools.descartes.dql.core.engine.whatif;

import tools.descartes.dql.lang.descartesQL.FilterigRule;
import tools.descartes.dql.lang.descartesQL.FilteringClause;

public class WhatIfFilterFactory {

	public static WhatIfFilter parseFilter(FilteringClause clause) {
		if (clause.getFilteringStrategy() != null)
			return parseFilter(clause.getFilteringStrategy().getFilteringStrategy());
		if (clause.getFilterungRule() != null) {
			return parseFilter(clause.getFilterungRule());
		}
		throw new IllegalStateException("Illegal filtering strategy");
	}

	private static WhatIfFilter parseFilter(FilterigRule filteringRule) {

		int filteringInt = filteringRule.getFilteringValue().getIntValue();
		double filteringDouble = filteringRule.getFilteringValue().getDoubleValue();

		if (filteringInt != 0 && filteringDouble == 0.0)
			if (filteringRule.getModifier() != null && filteringRule.getModifier().equals("%"))
				return new DoubleFilter((double) filteringInt, true);
			else
				return new DoubleFilter((double) filteringInt, false);
		if (filteringDouble != 0.0 && filteringInt == 0)
			if (filteringRule.getModifier() != null && filteringRule.getModifier().equals("%"))
				return new DoubleFilter(filteringDouble, true);
			else
				return new DoubleFilter(filteringDouble, false);

		throw new IllegalArgumentException("No valid filter found");

	}

	private static WhatIfFilter parseFilter(String filteringStrategy) {
		if (filteringStrategy.matches("ENTITY .+"))
			return new EntityFilter(filteringStrategy.substring(7));
		if (filteringStrategy.matches("METRIC .+"))
			return new MetricFilter(filteringStrategy.substring(7));
		if (filteringStrategy.matches("ONLY 0") || filteringStrategy.matches("0 ONLY") || filteringStrategy.matches("0") || filteringStrategy.matches("0 VALUES")
				|| filteringStrategy.matches("DEFAULT"))
			return new ZeroFilter();
		return null;
	}
}

