
package tools.descartes.dql.core.engine.whatif;

import java.util.LinkedList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;

import tools.descartes.dql.connector.ConnectorProvider;
import tools.descartes.dql.connector.ModelStructureQueryConnector;
import tools.descartes.dql.connector.PerformanceMetricsQueryConnector;
import tools.descartes.dql.connector.QueryConnector;
import tools.descartes.dql.core.engine.Activator;
import tools.descartes.dql.core.registry.ConnectorRegistry;
import tools.descartes.dql.lang.descartesQL.AliasClause;
import tools.descartes.dql.lang.descartesQL.ArithmeticClause;
import tools.descartes.dql.lang.descartesQL.DoFVariationClause;
import tools.descartes.dql.lang.descartesQL.EntityReference;
import tools.descartes.dql.lang.descartesQL.IntervalVariationClause;
import tools.descartes.dql.lang.descartesQL.MetricClause;
import tools.descartes.dql.lang.descartesQL.MetricClauseType;
import tools.descartes.dql.lang.descartesQL.MetricReference;
import tools.descartes.dql.lang.descartesQL.MetricReferenceClauses;
import tools.descartes.dql.lang.descartesQL.MultipleArithmeticClause;
import tools.descartes.dql.lang.descartesQL.ValueVariationClause;
import tools.descartes.dql.lang.descartesQL.WhatIfQuery;
import tools.descartes.dql.lang.descartesQL.WhatIfVariationClause;
import tools.descartes.dql.models.mapping.domain.DomainFactory;
import tools.descartes.dql.models.mapping.domain.Entity;
import tools.descartes.dql.models.mapping.domain.Probe;
import tools.descartes.dql.models.mapping.domain.Resource;
import tools.descartes.dql.models.mapping.domain.Service;
import tools.descartes.dql.models.mapping.mapping.EntityMapping;

public class WhatIfUtil {

	private static final DomainFactory domainFactory = DomainFactory.eINSTANCE;

	public static PerformanceMetricsQueryConnector getPmqc(String family) {
		ConnectorProvider provider = getConnectorProvider(family, PerformanceMetricsQueryConnector.class);
		return provider.createPerformanceMetricsQueryConnectorInstance();
	}

	public static ModelStructureQueryConnector getMsqc(String family) {
		ConnectorProvider provider = getConnectorProvider(family, ModelStructureQueryConnector.class);
		return provider.createModelStructureQueryConnectorInstance();
	}

	private static ConnectorProvider getConnectorProvider(String family, Class<? extends QueryConnector> connectorClass) {
		BundleContext context = Activator.getContext();
		ServiceReference<ConnectorRegistry> registryRef = context.getServiceReference(ConnectorRegistry.class);
		ConnectorRegistry registry = context.getService(registryRef);
		ServiceReference<ConnectorProvider> providerRef = registry.requestServiceReference(family, connectorClass);
		return context.getService(providerRef);
	}

	/*
	 * Existing Method reused. Have a look on
	 * PerformanceMetricsQueryInterpreterImpl
	 */
	public static void processSelectClause(final WhatIfQuery whatIfQuery, final Map<String, Entity> aliasMap) {
		final MetricReferenceClauses metricRefClauses = whatIfQuery.getMetricReferenceClauses();
		final EList<MetricClauseType> metricClauseTypes = metricRefClauses.getMetricClauseTypes();

		for (final MetricClauseType ref : metricClauseTypes) {
			final Probe probe = domainFactory.createProbe();
			Entity entity = null;

			if (ref instanceof MetricClause) {
				// Found a plain metric
				final MetricClause metricClause = (MetricClause) ref;

				final MetricReference metricRef = metricClause.getMetricReference();
				final String entityRef = metricRef.getEntityReference();
				final String metricRef2 = metricRef.getMetricReference();

				entity = aliasMap.get(entityRef);
				probe.setMetricName(metricRef2);
			} else {
				throw new IllegalArgumentException("Unsupported clause detected: " + ref);
			}

			// Naming metrics from undefined aliases makes little sense
			if (entity == null) {
				throw new IllegalArgumentException("Missing entity alias for requested metric, aborting.");
			} else {
				final EList<Probe> probes = entity.getProbes();
				probes.add(probe);
			}
		}
	}

	/*
	 * Existing Method reused. Have a look on
	 * PerformanceMetricsQueryInterpreterImpl
	 */
	public static void processForClause(final WhatIfQuery WhatIfQuery, final Map<String, Entity> aliasMap, final EntityMapping request) {
		final List<Resource> resources = request.getResources();
		final List<Service> services = request.getServices();
		final List<EntityReference> entityRefs = WhatIfQuery.getForClause().getEntityReferenceClause().getEntityReferences();

		for (final EntityReference ref : entityRefs) {
			Entity entity = null;

			switch (ref.getType()) {
			case RESOURCE:
				entity = domainFactory.createResource();
				resources.add((Resource) entity);
				break;
			case SERVICE:
				entity = domainFactory.createService();
				services.add((Service) entity);
				break;
			default:
				throw new IllegalArgumentException("Invalid entity type found, aborting.");
			}

			final AliasClause aliasClause = ref.getAlias();

			if (aliasClause != null) {
				entity.setAlias(aliasClause.getAlias());
			}

			entity.setIdentifier(ref.getIdentifier());

			if (aliasMap.put(entity.getAlias(), entity) != null) {
				throw new IllegalArgumentException("Duplicate aliases in FOR, aborting.");
			}
		}
	}

	/*
	 * schöner machen möglich :)
	 */
	/*
	 * generating List of values, starting with a state value
	 */
	public static HashMap<String, List<Integer>> processWhatIfVariation(final WhatIfQuery whatIfQuery, final Map<String, Integer> initialValues) {

		LinkedList<WhatIfVariationClause> numberOfVariables = new LinkedList<WhatIfVariationClause>(whatIfQuery.getWhatIfVariation().getWhatIfVariationClause());

		HashMap<String, List<Integer>> results = new HashMap<String, List<Integer>>();

		for (int i = 0; i < numberOfVariables.size(); i++) {

			EObject dof = numberOfVariables.get(i).getValues().getDofVariationClause();
			EObject multi = numberOfVariables.get(i).getValues().getMultipleArithmethicClause();

			LinkedList<Integer> newValues = new LinkedList<Integer>();
			newValues.add(initialValues.get(numberOfVariables.get(i).getVariationPoint()));

			// Variation with DoF clauses
			if (dof != null) {
				final EObject dofVariation = ((DoFVariationClause) dof).getVariation();
				if (dofVariation instanceof ValueVariationClause) {
					ValueVariationClause clause = (ValueVariationClause) dofVariation;

					if (clause.getValue().contains(initialValues.get(numberOfVariables.get(i).getVariationPoint()))) {
						clause.getValue().remove(initialValues.get(numberOfVariables.get(i).getVariationPoint()));
					}
					newValues.addAll(clause.getValue());
					results.put((numberOfVariables.get(i).getVariationPoint()), newValues);

				} else if (dofVariation instanceof IntervalVariationClause) {
					IntervalVariationClause interval = (IntervalVariationClause) dofVariation;
					int stop = interval.getEnd();
					int step = interval.getStep();
					int value = interval.getStart();
					while (value <= stop) {
						if (!newValues.contains(value)) {
							newValues.add(value);
						}
						value = value + step;
					}

					if (newValues.get(newValues.size() - 1) != stop) {
						if (!newValues.contains(value))
							newValues.add(stop);
					}

					if (newValues.get(newValues.size() - 1) == newValues.get(0)) {
						newValues.remove(newValues.size() - 1);
					}

					results.put((numberOfVariables.get(i).getVariationPoint()), newValues);
				}
			}

			// variation with multiple Arithmetic clauses
			if (multi != null) {

				LinkedList<ArithmeticClause> listOfClauses = new LinkedList<ArithmeticClause>();
				listOfClauses.addAll(((MultipleArithmeticClause) multi).getMultiClause());

				for (int j = 0; j < listOfClauses.size(); j++) {
					int newValue = 0;
					ArithmeticClause arithmeticClause = listOfClauses.get(j);

					if (arithmeticClause.getMode().equals("ADD")) {
						if (arithmeticClause.getValue().getIntValue() == 0) {
							newValue = (int) (initialValues.get(numberOfVariables.get(i).getVariationPoint()) + arithmeticClause.getValue().getDoubleValue());
							newValues.add(newValue);
						} else {
							newValue = initialValues.get(numberOfVariables.get(i).getVariationPoint()) + arithmeticClause.getValue().getIntValue();
							newValues.add(newValue);
						}

					} else if (arithmeticClause.getMode().equals("SUB")) {
						if (arithmeticClause.getValue().getIntValue() == 0) {
							newValue = (int) (initialValues.get(numberOfVariables.get(i).getVariationPoint()) - arithmeticClause.getValue().getDoubleValue());
							System.out.println(newValue);
							if (newValue < 0)
								throw new IllegalArgumentException("Illegal Argument for variationPoint " + numberOfVariables.get(i).getVariationPoint()
										+ " with subtraction of value " + arithmeticClause.getValue().getDoubleValue() + " with result of " + newValue);
							newValues.add(newValue);
						} else {
							newValue = initialValues.get(numberOfVariables.get(i).getVariationPoint()) - arithmeticClause.getValue().getIntValue();
							System.out.println(newValue);
							if (newValue < 0)
								throw new IllegalArgumentException("Illegal Argument for variationPoint " + numberOfVariables.get(i).getVariationPoint()
										+ " with subtraction of value " + arithmeticClause.getValue().getIntValue() + " with result of " + newValue);

							newValues.add(newValue);
						}

					} else if (arithmeticClause.getMode().equals("MULTIPLY")) {
						if (arithmeticClause.getValue().getIntValue() == 0) {
							newValue = (int) (initialValues.get(numberOfVariables.get(i).getVariationPoint()) * arithmeticClause.getValue().getDoubleValue());
							newValues.add(newValue);
						} else {
							newValue = initialValues.get(numberOfVariables.get(i).getVariationPoint()) * arithmeticClause.getValue().getIntValue();
							newValues.add(newValue);
						}

					} else
						throw new IllegalArgumentException("Illegal operation at variationPoint " + numberOfVariables.get(i).getVariationPoint());

					results.put(numberOfVariables.get(i).getVariationPoint(), newValues);

				}
			}

		}

		return results;
	}
}
