/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.dql.core.platform.eclipse.views;

import java.math.BigDecimal;

import org.eclipse.swt.SWT;

public class TableRowBean implements Comparable<TableRowBean> {
	public static final String VALUE_NOT_SET = "";

	public static final String TYPE_HEADING = "Entity Type";
	public static final String ALIAS_HEADING = "Entity Alias";
	public static final String IDENTIFIER_HEADING = "Entity Identifier";
	public static final String METRIC_HEADING = "Metric Name";
	public static final String VALID_HEADING = "Result Valid?";
	public static final String VALUE_HEADING = "Result Value";
	public static final String ACCURACY_HEADING = "Result Accuracy";

	public static final String[] HEADINGS = new String[] { TYPE_HEADING, ALIAS_HEADING, IDENTIFIER_HEADING,
			METRIC_HEADING, VALID_HEADING, VALUE_HEADING, ACCURACY_HEADING };

	public static final int TYPE_WIDTH = 100;
	public static final int ALIAS_WIDTH = 200;
	public static final int IDENTIFIER_WIDTH = 200;
	public static final int METRIC_WIDTH = 200;
	public static final int VALID_WIDTH = 100;
	public static final int VALUE_WIDTH = 150;
	public static final int ACCURACY_WIDTH = 100;

	public static final int[] WIDTHS = new int[] { TYPE_WIDTH, ALIAS_WIDTH, IDENTIFIER_WIDTH, METRIC_WIDTH,
			VALID_WIDTH, VALUE_WIDTH, ACCURACY_WIDTH };

	public static final int TOTAL_WIDTH = sum(WIDTHS);

	public static final int TYPE_POS = 0;
	public static final int ALIAS_POS = 1;
	public static final int IDENTIFIER_POS = 2;
	public static final int METRIC_POS = 3;
	public static final int VALID_POS = 4;
	public static final int VALUE_POS = 5;
	public static final int ACCURACY_POS = 6;

	public static final int TYPE_ALIGNMENT = SWT.LEFT;
	public static final int ALIAS_ALIGNMENT = SWT.LEFT;
	public static final int IDENTIFIER_ALIGNMENT = SWT.LEFT;
	public static final int METRIC_ALIGNMENT = SWT.LEFT;
	public static final int VALID_ALIGNMENT = SWT.LEFT;
	public static final int VALUE_ALIGNMENT = SWT.RIGHT;
	public static final int ACCURACY_ALIGNMENT = SWT.RIGHT;

	static final int[] WEIGHTS = new int[] { weight(WIDTHS[TYPE_POS]), weight(WIDTHS[ALIAS_POS]),
			weight(WIDTHS[IDENTIFIER_POS]), weight(WIDTHS[METRIC_POS]), weight(WIDTHS[VALID_POS]),
			weight(WIDTHS[VALUE_POS]), weight(WIDTHS[ACCURACY_POS]) };

	public static final int[] ALIGNMENT = new int[] { TYPE_ALIGNMENT, ALIAS_ALIGNMENT, IDENTIFIER_ALIGNMENT,
			METRIC_ALIGNMENT, VALID_ALIGNMENT, VALUE_ALIGNMENT, ACCURACY_ALIGNMENT };

	private static int sum(final int[] widths2) {
		int totalWidth = 0;

		for (final int width : widths2) {
			totalWidth += width;
		}

		return totalWidth;
	}

	private static int weight(final int width) {
		return (int) ((double) (width * 100) / (double) TOTAL_WIDTH);
	}

	private String type = VALUE_NOT_SET;
	private String alias = VALUE_NOT_SET;
	private String identifier = VALUE_NOT_SET;
	private String metricName = VALUE_NOT_SET;
	private String valid = VALUE_NOT_SET;
	private String value = VALUE_NOT_SET;

	private String accuracy = VALUE_NOT_SET;

	public static final String DOF_ICON = "icons/dof.png";

	public static final String AGGREGATE_ICON = "icons/aggregate.png";

	public static final String RESOURCE_ICON = "icons/resource.png";

	public static final String SERVICE_ICON = "icons/service.png";

	public static final int DECIMAL_ROUNDING = BigDecimal.ROUND_HALF_EVEN;

	public static final int DECIMAL_SCALE = 6;

	public static final String RESOURCE = "Resource";

	public static final String SERVICE = "Service";

	public static final String DOF = "DoF";

	public static final String AGGREGATE = "Aggregate";

	public static final String INVALID_TYPE = "Oops, unknown type :-)";

	public static final String SERVICE_ICON_KEY = "icon.service";

	public static final String RESOURCE_ICON_KEY = "icon.resource";

	public static final String AGGREGATE_ICON_KEY = "icon.aggregate";

	public static final String DOF_ICON_KEY = "icon.dof";

	public TableRowBean() {
	}

	public TableRowBean(final String type, final String alias, final String identifier) {
		super();
		this.type = type;
		this.alias = alias;
		this.identifier = identifier;
	}

	public TableRowBean(final String type, final String alias, final String identifier, final String metricName,
			final String valid, final String value, final String accuracy) {
		super();
		this.type = type;
		this.alias = alias;
		this.identifier = identifier;
		this.metricName = metricName;
		this.valid = valid;
		this.value = value;
		this.accuracy = accuracy;
	}

	@Override
	public int compareTo(final TableRowBean otherTableRowBean) {
		final String otherType = otherTableRowBean.getType();
		final int typeComparison = type.compareTo(otherType);

		if (typeComparison != 0) {
			return typeComparison;
		}

		final String otherAlias = otherTableRowBean.getAlias();
		final int aliasComparison = alias.compareTo(otherAlias);

		if (aliasComparison != 0) {
			return aliasComparison;
		}

		final String otherMetricName = otherTableRowBean.getMetricName();
		final int metricNameComparison = metricName.compareTo(otherMetricName);

		if (metricNameComparison != 0) {
			return metricNameComparison;
		}

		return 0;
	}

	public String getAccuracy() {
		return accuracy;
	}

	public String getAlias() {
		return alias;
	}

	public String getIdentifier() {
		return identifier;
	}

	public String getMetricName() {
		return metricName;
	}

	public String getType() {
		return type;
	}

	public String getValid() {
		return valid;
	}

	public String getValue() {
		return value;
	}

	public void setAccuracy(final String accuracy) {
		this.accuracy = accuracy;
	}

	public void setAlias(final String alias) {
		this.alias = alias;
	}

	public void setIdentifier(final String identifier) {
		this.identifier = identifier;
	}

	public void setMetricName(final String metricName) {
		this.metricName = metricName;
	}

	public void setType(final String type) {
		this.type = type;
	}

	public void setValid(final String valid) {
		this.valid = valid;
	}

	public void setValue(final String value) {
		this.value = value;
	}

}
