/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.dql.core.platform.eclipse.views;

import static tools.descartes.dql.core.platform.eclipse.views.TableRowBean.AGGREGATE;
import static tools.descartes.dql.core.platform.eclipse.views.TableRowBean.AGGREGATE_ICON_KEY;
import static tools.descartes.dql.core.platform.eclipse.views.TableRowBean.DOF;
import static tools.descartes.dql.core.platform.eclipse.views.TableRowBean.DOF_ICON_KEY;
import static tools.descartes.dql.core.platform.eclipse.views.TableRowBean.RESOURCE;
import static tools.descartes.dql.core.platform.eclipse.views.TableRowBean.RESOURCE_ICON_KEY;
import static tools.descartes.dql.core.platform.eclipse.views.TableRowBean.SERVICE;
import static tools.descartes.dql.core.platform.eclipse.views.TableRowBean.SERVICE_ICON_KEY;

import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;

import tools.descartes.dql.core.platform.eclipse.Activator;

class ViewLabelProvider extends LabelProvider implements ITableLabelProvider {

	private final ImageRegistry reg = Activator.getPlugin().getImageRegistry();

	@Override
	public Image getColumnImage(final Object obj, final int index) {
		if (index != 0) {
			return null;
		}

		final TableRowBean tableRow = (TableRowBean) obj;
		final String type = tableRow.getType();

		if (type.equals(SERVICE)) {
			return reg.get(SERVICE_ICON_KEY);
		} else if (type.equals(RESOURCE)) {
			return reg.get(RESOURCE_ICON_KEY);
		} else if (type.equals(AGGREGATE)) {
			return reg.get(AGGREGATE_ICON_KEY);
		} else if (type.equals(DOF)) {
			return reg.get(DOF_ICON_KEY);
		}

		return getImage(obj);
	}

	@Override
	public String getColumnText(final Object obj, final int index) {
		final TableRowBean row = (TableRowBean) obj;

		switch (index) {
		case TableRowBean.TYPE_POS:
			return row.getType();
		case TableRowBean.ALIAS_POS:
			return row.getAlias();
		case TableRowBean.IDENTIFIER_POS:
			return row.getIdentifier();
		case TableRowBean.METRIC_POS:
			return row.getMetricName();
		case TableRowBean.VALID_POS:
			return row.getValid();
		case TableRowBean.VALUE_POS:
			return row.getValue();
		case TableRowBean.ACCURACY_POS:
			return row.getAccuracy();
		}

		return "Unknown Column Index...";
	}

	@Override
	public Image getImage(final Object obj) {
		return PlatformUI.getWorkbench().getSharedImages().getImage(ISharedImages.IMG_OBJ_ELEMENT);
	}
}
