/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.dql.vizualization.observations;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.log4j.Logger;

import tools.descartes.dql.lang.descartesQL.AbsoluteTimeClause;
import tools.descartes.dql.lang.descartesQL.TimeClause;
import tools.descartes.dql.vizualization.util.DQLLogger;

public class AbsoluteTimeCalculator implements TimeCalculator {
	public final String DATE_DAY = "yyyy-MM-dd";
	public final String DATE_SEC = "yyyy-MM-dd'T'HH:mm:ss";
	public final String DATE_SEC_TZ = "yyyy-MM-dd'T'HH:mm:ssZ";
	public final String DATE_MSEC = "yyyy-MM-dd'T'HH:mm:ss.SSS";
	public final String DATE_MSEC_TZ = "yyyy-MM-dd'T'HH:mm:ss.SSSZ";

	public final String[] DATE_FORMATS = new String[] { DATE_MSEC_TZ, DATE_MSEC, DATE_SEC_TZ, DATE_SEC, DATE_DAY };

	private final Logger log = DQLLogger.getLogger(this.getClass().getName());
	private Date date = null;
	private boolean valid = false;

	public AbsoluteTimeCalculator() {
		super();
	}

	public AbsoluteTimeCalculator(final TimeClause timeSpecification) {
		parseDate(timeSpecification);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * tools.descartes.dql.core.engine.observations.TimeCalculator#getDate
	 * ()
	 */
	@Override
	public Date getDate() {
		return date;
	}

	@Override
	public boolean isBackwards() {
		return false;
	}

	@Override
	public boolean isValid() {
		return valid;
	}

	@Override
	public boolean parseDate(final TimeClause timeClause) {
		final AbsoluteTimeClause absoluteTimeClause = (AbsoluteTimeClause) timeClause;
		final String timeSpecification = absoluteTimeClause.getTimeSpec();
		final SimpleDateFormat dateFormat = new SimpleDateFormat();
		Date parsedDate = null;

		// Use strict parsing!
		dateFormat.setLenient(false);

		for (final String element : DATE_FORMATS) {
			dateFormat.applyPattern(element);

			try {
				parsedDate = dateFormat.parse(timeSpecification);
			} catch (final ParseException e) {
				log.info(e.getMessage());
			}

			// Got a date, we're done
			if (parsedDate != null) {
				date = parsedDate;
				valid = true;
				return true;
			}
		}

		return false;
	}

	@Override
	public void setBackwards(final boolean backwards) {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see tools.descartes.dql.core.engine.observations.TimeCalculator#
	 * setReferenceDate(java.util.Date)
	 */
	@Override
	public void setReferenceDate(final Date referenceDate) {
		// As this class is used to work on AbsoluteTimeClauses, we do not need
		// a reference date
	}

}
