/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.dql.vizualization.observations;

import java.util.Date;

import tools.descartes.dql.connector.PerformanceMetricsQueryConnector;
import tools.descartes.dql.lang.descartesQL.AbsoluteTimeClause;
import tools.descartes.dql.lang.descartesQL.ObserveRelativeClause;
import tools.descartes.dql.lang.descartesQL.ObserveRelativeDirectionType;
import tools.descartes.dql.lang.descartesQL.TimeClause;

public class ObserveRelativeInterpreter implements ObservationInterpreter {
	private final ObserveRelativeClause observation;
	private boolean referenceTimeAsEnd = true;
	private final Date referenceDate = new Date();
	private Date otherTime = null;

	public ObserveRelativeInterpreter(final ObserveRelativeClause observation) {
		this.observation = observation;
		interpretDate();
	}

	@Override
	public void configureConnector(final PerformanceMetricsQueryConnector queryConnector, final long sampleSize) {
		if (referenceTimeAsEnd) {
			queryConnector.setObservationPeriod(otherTime, referenceDate, sampleSize);
		} else {
			queryConnector.setObservationPeriod(referenceDate, otherTime, sampleSize);
		}

	}

	@Override
	public void interpretDate() {
		final ObserveRelativeDirectionType direction = observation.getDirection();
		final TimeClause timeTarget = observation.getTimeTarget();

		if (direction.equals(ObserveRelativeDirectionType.SINCE)) {
			referenceTimeAsEnd = true;
		} else {
			referenceTimeAsEnd = false;
		}

		TimeCalculator calc = null;

		if (timeTarget instanceof AbsoluteTimeClause) {
			calc = new AbsoluteTimeCalculator(timeTarget);
		} else {
			calc = new RelativeTimeCalculator();
			calc.setBackwards(referenceTimeAsEnd);
			calc.parseDate(timeTarget);
		}

		if (calc.isValid()) {
			otherTime = calc.getDate();
		}
	}

}
