/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.dql.vizualization.observations;

import static tools.descartes.dql.vizualization.observations.TimeModifierConverter.convert;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import tools.descartes.dql.lang.descartesQL.RelativeTimeClause;
import tools.descartes.dql.lang.descartesQL.RelativeTimeDurationClause;
import tools.descartes.dql.lang.descartesQL.TimeClause;
import tools.descartes.dql.lang.descartesQL.TimeModifierType;

public class RelativeTimeCalculator implements TimeCalculator {
	private final boolean valid = false;

	private boolean backwards = false;
	private long millis = 0;
	private Date referenceDate;

	public RelativeTimeCalculator() {
		super();
	}

	@Override
	public Date getDate() {
		final Calendar calendar = new GregorianCalendar();

		if (referenceDate != null) {
			calendar.setTime(referenceDate);
		} else {
			// Ensure reentrant behavior
			referenceDate = calendar.getTime();
		}

		final long targetedTime = calendar.getTimeInMillis() + millis;
		calendar.setTimeInMillis(targetedTime);

		final Date targetedDate = calendar.getTime();
		return targetedDate;
	}

	@Override
	public boolean isBackwards() {
		return backwards;
	}

	@Override
	public boolean isValid() {
		return valid;
	}

	@Override
	public boolean parseDate(final TimeClause timeClause) {
		final RelativeTimeClause relativeTimeClause = (RelativeTimeClause) timeClause;
		millis = 0;

		for (final RelativeTimeDurationClause r : relativeTimeClause.getDurationClause()) {
			final int duration = r.getDuration();
			final TimeModifierType timeModifier = r.getTimeModifier();

			millis += duration * convert(timeModifier);
		}

		if (backwards == true) {
			millis *= (-1l);
		}

		return true;
	}

	@Override
	public void setBackwards(final boolean backwards) {
		// If the value switched the direction, invert it
		if (this.backwards ^ backwards) {
			millis *= (-1l);
		}

		this.backwards = backwards;
	}

	@Override
	public void setReferenceDate(final Date referenceDate) {
		this.referenceDate = referenceDate;
	}
}
