/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.dql.vizualization.observations;

import static tools.descartes.dql.lang.descartesQL.TimeModifierType.DAY;
import static tools.descartes.dql.lang.descartesQL.TimeModifierType.HOUR;
import static tools.descartes.dql.lang.descartesQL.TimeModifierType.MILLISECOND;
import static tools.descartes.dql.lang.descartesQL.TimeModifierType.MINUTE;
import static tools.descartes.dql.lang.descartesQL.TimeModifierType.MONTH;
import static tools.descartes.dql.lang.descartesQL.TimeModifierType.SECOND;
import static tools.descartes.dql.lang.descartesQL.TimeModifierType.WEEK;
import tools.descartes.dql.lang.descartesQL.TimeModifierType;

/**
 * Calculates the factor for a given time modifier
 * 
 * @author Fabian Gorsler <fabian.gorsler@student.kit.edu>
 */
public class TimeModifierConverter {
	public static final long DAY_MS = 24l * convert(HOUR);
	public static final long HOUR_MS = 60l * convert(MINUTE);
	public static final long MINUTE_MS = 60l * convert(SECOND);

	/**
	 * Assume each month consists of four WEEK_MS
	 */
	public static final long MONTH_MS = 4l * convert(WEEK);

	public static final long SECOND_MS = 1000l * convert(MILLISECOND);
	public static final long WEEK_MS = 7l * convert(DAY);
	public static final long YEAR_MS = 12l * convert(MONTH);
	public static final long NULL_MS = 0l;
	public static final long ONE_MS = 1l;

	/**
	 * Conversion of a TimeModifierType into its representation as milliseconds.
	 * The conversion is based on the preceding (i.e. smaller) time unit with
	 * recursion until the smallest time unit supported (i.e. milliseconds) is
	 * reached. The recursion can be resolved by the compiler.
	 * 
	 * @param modifier
	 * @return
	 */
	public static long convert(final TimeModifierType modifier) {
		switch (modifier) {
		case DAY:
			return DAY_MS;
		case HOUR:
			return HOUR_MS;
		case MILLISECOND:
			return ONE_MS;
		case MINUTE:
			return MINUTE_MS;
		case MONTH:
			return MONTH_MS;
		case SECOND:
			return SECOND_MS;
		case WEEK:
			return WEEK_MS;
		case YEAR:
			return YEAR_MS;
		default:
			return NULL_MS;
		}
	}
}
